      logical function pass_point(p)
c************************************************************************
c     This function is called from sample to see if it needs to 
c     bother calculating the weight from all the different conficurations
c     You can either just return true, or have it call passcuts
c************************************************************************
      implicit none
c
c     Arguments
c
      double precision p
c
c     External
c
      logical passcuts
      external passcuts
c-----
c  Begin Code
c-----
      pass_point = .true.
c      pass_point = passcuts(p)
      end


      LOGICAL FUNCTION PASSCUTS(P,rwgt)
C**************************************************************************
C     INPUT:
C            P(0:3,1)           MOMENTUM OF INCOMING PARTON
C            P(0:3,2)           MOMENTUM OF INCOMING PARTON
C            P(0:3,3)           MOMENTUM OF d
C            P(0:3,4)           MOMENTUM OF b
C            P(0:3,5)           MOMENTUM OF bbar
C            P(0:3,6)           MOMENTUM OF e+
C            P(0:3,7)           MOMENTUM OF ve
C            COMMON/JETCUTS/   CUTS ON JETS
C     OUTPUT:
C            TRUE IF EVENTS PASSES ALL CUTS LISTED
C**************************************************************************
      IMPLICIT NONE
c
c     Constants
c
      include 'genps.inc'
C
C     ARGUMENTS
C
      REAL*8 P(0:3,nexternal),rwgt

C
C     LOCAL
C
      LOGICAL FIRSTTIME,FIRSTTIME2,pass_bw, notgood
      integer i,j
      REAL*8 XVAR,hardj1,hardj2,ptmax1,ptmax2
C
C     EXTERNAL
C
      REAL*8 R2,DOT,ET,ETA,DJ,SumDot,pt,rewgt
      logical cut_bw
      external cut_bw,rewgt
C
C     GLOBAL
C
      include 'run.inc'

      double precision etmin(nincoming+1:nexternal),etamax(nincoming+1:nexternal)
      double precision emin(nincoming+1:nexternal)
      double precision                    r2min(nincoming+1:nexternal,nincoming+1:nexternal)
      double precision s_min(nexternal,nexternal)

      common/to_cuts/  etmin, emin, etamax, r2min, s_min

c
c     Special cuts
c

      integer        lbw(0:nexternal)  !Use of B.W.
      common /to_BW/ lbw
C
C     SPECIAL CUTS
C
      REAL*8 XPTJ,XPTB,XPTA,XPTL
      REAL*8 XETAMIN,XQCUT,DELTAETA
      COMMON /TO_SPECXPT/XPTJ,XPTB,XPTA,XPTL,XETAMIN,XQCUT,DELTAETA
      LOGICAL  IS_A_J(NEXTERNAL),IS_A_L(NEXTERNAL)
      LOGICAL  IS_A_B(NEXTERNAL),IS_A_A(NEXTERNAL)
      COMMON /TO_SPECISA/IS_A_J,IS_A_A,IS_A_L,IS_A_B

      include 'coupl.inc'
C
C
c
      DATA FIRSTTIME,FIRSTTIME2/.TRUE.,.TRUE./

c put momenta in common block for couplings.f
      double precision PP(0:3,max_particles)
      COMMON /MOMENTA_PP/PP


C-----
C  BEGIN CODE
C-----

c      include 'coupl.inc'

      PASSCUTS=.TRUE.             !EVENT IS OK UNLESS OTHERWISE CHANGED
      IF (FIRSTTIME) THEN
         FIRSTTIME=.FALSE.
c
c
         write(*,'(a10,10i8)') 'Particle',(i,i=nincoming+1,nexternal)
         write(*,'(a10,10f8.1)') 'Et >',(etmin(i),i=nincoming+1,nexternal)
         write(*,'(a10,10f8.1)') 'E >',(emin(i),i=nincoming+1,nexternal)
         write(*,'(a10,10f8.1)') 'Eta <',(etamax(i),i=nincoming+1,nexternal)
         do j=nincoming+1,nexternal-1
            write(*,'(a,i2,a,10f8.1)') 'd R #',j,'  >',(-0.0,i=nincoming+1,j),
     &           (r2min(i,j),i=j+1,nexternal)
            do i=j+1,nexternal
               r2min(i,j)=r2min(i,j)*dabs(r2min(i,j))    !Since r2 returns distance squared
            enddo
         enddo
         do j=1,nexternal-1
            write(*,'(a,i2,a,10f8.1)') 's min #',j,'>',
     &           (s_min(i,j),i=nincoming+1,nexternal)
         enddo

c
c     Set the strong coupling
c
         call set_ren_scale(P,scale)
c
c     Check that the user funtions for setting the scales
c     have been edited if the choice of an event-by-event
c     scale choice has been made 
c
         if(.not.fixed_ren_scale) then
            if(scale.eq.0d0) then
               write(6,*) 
               write(6,*) '* >>>>>>>>>ERROR<<<<<<<<<<<<<<<<<<<<<<<*'
               write(6,*) ' Dynamical renormalization scale choice '
               write(6,*) ' selected but user subroutine' 
               write(6,*) ' set_ren_scale not edited in file:setpara.f'
               write(6,*) ' Switching to a fixed_ren_scale choice'
               write(6,*) ' with scale=zmass'
               scale=91.2d0
               write(6,*) 'scale=',scale
               fixed_ren_scale=.true.
               call set_ren_scale(P,scale)
            endif
         endif
         
         if(.not.fixed_fac_scale) then
            call set_fac_scale(P,q2fact)
            if(q2fact(1).eq.0d0.or.q2fact(2).eq.0d0) then
               write(6,*) 
               write(6,*) '* >>>>>>>>>ERROR<<<<<<<<<<<<<<<<<<<<<<<*'
               write(6,*) ' Dynamical renormalization scale choice '
               write(6,*) ' selected but user subroutine' 
               write(6,*) ' set_fac_scale not edited in file:setpara.f'
               write(6,*) ' Switching to a fixed_fac_scale choice'
               write(6,*) ' with q2fact(i)=zmass**2'
               fixed_fac_scale=.true.
               q2fact(1)=91.2d0**2
               q2fact(2)=91.2d0**2
               write(6,*) 'scales=',q2fact(1),q2fact(2)
            endif
         endif

         if(fixed_ren_scale) then
            call setpara('param_card.dat',.false.)
         endif

c      Preparation for reweighting by setting up clustering by diagrams
         call initcluster()

c     Put momenta in the common block to zero to start
         do i=0,3
            do j=1,max_particles
               pp(i,j) = 0d0
            enddo
         enddo
         
      ENDIF
c
c     Make sure have reasonable 4-momenta
c
      if (p(0,1) .le. 0d0) then
         passcuts=.false.
         return
      endif

c     Also make sure there's no INF or NAN
      do i=1,nexternal
         do j=0,3
            if(p(j,i).gt.1d32.or.p(j,i).ne.p(j,i))then
               passcuts=.false.
               return
            endif
         enddo
      enddo
c
c     Here we reset factorization and renormalization
c     scales on an event-by-event basis
c
      if(.not.fixed_ren_scale) then
         call set_ren_scale(P,scale)
      endif

      if(.not.fixed_fac_scale) then
         call set_fac_scale(P,q2fact)
      endif

c
c     Limit S_hat
c
c      if (x1*x2*stot .gt. 500**2) then
c         passcuts=.false.
c         return
c      endif
      if (.true.) then
c
c     p_t cuts
c
         do i=nincoming+1,nexternal
            if(etmin(i).ge.0d0) then
               notgood=pt(p(0,i)) .le. etmin(i)
            else
               notgood=pt(p(0,i)) .gt. -etmin(i)
            endif
            if (notgood) then
               passcuts=.false.
               return
            endif
         enddo
c
c     E cuts
c
         do i=nincoming+1,nexternal
            if(emin(i).ge.0d0) then
               notgood=p(0,i) .le. emin(i)
            else
               notgood=p(0,i) .gt. -emin(i)
            endif
            if (notgood) then
               passcuts=.false.
               return
            endif
         enddo
c
c     Pseudo-rapidity cuts
c
      do i=nincoming+1,nexternal
            if(etamax(i).ge.0d0) then
               notgood=abs(eta(p(0,i))) .gt. etamax(i)
            else
               notgood=abs(eta(p(0,i))) .lt. -etamax(i)
            endif
            if (notgood) then
               passcuts=.false.
               return
            endif
      enddo
c
c     delta-R cuts
c
      do i=nincoming+1,nexternal-1
         do j=i+1,nexternal
            if(r2min(j,i).ge.0d0) then
               notgood=r2(p(0,i),p(0,j)) .lt. r2min(j,i)
            else
               notgood=r2(p(0,i),p(0,j)) .gt. -r2min(j,i)
            endif
            if (notgood) then
               passcuts=.false.
               return
            endif
         enddo
      enddo

      endif
c
c     t-channel invarient mass cuts
c
c      do i=1,2
c         do j=nincoming+1,nexternal
c            if (abs(SumDot(p(0,i),p(0,j),-1d0)) .lt. s_min(j,i)) then
c               passcuts=.false.
c               return
c            endif
c         enddo
c      enddo
c
c     s-channel invarient mass cuts
c
      do i=nincoming+1,nexternal-1
         do j=i+1,nexternal
            if(s_min(j,i).ge.0d0) then
               notgood=SumDot(p(0,i),p(0,j),+1d0) .lt. s_min(j,i)
            else
               notgood=SumDot(p(0,i),p(0,j),+1d0) .gt. -s_min(j,i)
            endif
            if (notgood) then
               passcuts=.false.
               return
            endif
         enddo
      enddo


c
c     B.W. phase space cuts
c
      pass_bw=cut_bw(p)
      if (lbw(0) .eq. 1) then
        if ( pass_bw ) then
          passcuts=.false.
c       write(*,*) 'Failed BW',sqrt(2d0*dot(p(0,3),p(0,4)))
          return
        endif
      endif


C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>     
C     SPECIAL CUTS
C<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

C     REQUIRE AT LEAST ONE JET WITH PT>XPTJ
         
         IF(xptj.gt.0d0) THEN
            xvar=0
            do i=nincoming+1,nexternal
               if(is_a_j(i)) xvar=max(xvar,pt(p(0,i)))
            enddo
            if (xvar .lt. xptj) then
               passcuts=.false.
               return
            endif
         ENDIF

C     REQUIRE AT LEAST ONE PHOTON WITH PT>XPTA
         
         IF(xpta.gt.0d0) THEN
            xvar=0
            do i=nincoming+1,nexternal
               if(is_a_a(i)) xvar=max(xvar,pt(p(0,i)))
            enddo
            if (xvar .lt. xpta) then
               passcuts=.false.
               return
            endif
         ENDIF

C     REQUIRE AT LEAST ONE B  WITH PT>XPTB
         
         IF(xptb.gt.0d0) THEN
            xvar=0
            do i=nincoming+1,nexternal
               if(is_a_b(i)) xvar=max(xvar,pt(p(0,i)))
            enddo
            if (xvar .lt. xptb) then
               passcuts=.false.
               return
            endif
         ENDIF

C     REQUIRE AT LEAST ONE LEPTON  WITH PT>XPTL
         
         IF(xptl.gt.0d0) THEN
            xvar=0
            do i=nincoming+1,nexternal
               if(is_a_l(i)) xvar=max(xvar,pt(p(0,i)))
            enddo
            if (xvar .lt. xptl) then
               passcuts=.false.
               return
            endif
         ENDIF
C
C     WBF CUTS: TWO TYPES
C    
C     FIRST TYPE:  implemented by FM
C
C     1. FIND THE 2 HARDEST JETS
C     2. REQUIRE |ETA(J)|>XETAMIN
C     3. REQUIRE ETA(J1)*ETA(J2)<0
C
C     SECOND TYPE : added by Simon de Visscher 1-08-2007
C
C     1. FIND THE 2 HARDEST JETS
C     2. REQUIRE |ETA(J1)-ETA(J2)|>DELTAETA
C     3. REQUIRE ETA(J1)*ETA(J2)<0
C
C
         hardj1=0
         hardj2=0
         ptmax1=0d0
         ptmax2=0d0

C-- START IF AT LEAST ONE OF THE CUTS IS ACTIVATED
         
         IF(XETAMIN.GT.0D0.OR.DELTAETA.GT.0D0) THEN
            
C-- FIND THE HARDEST JETS

            do i=nincoming+1,nexternal
               if(is_a_j(i)) then
c                  write (*,*) i,pt(p(0,i))
                  if(pt(p(0,i)).gt.ptmax1) then
                     hardj2=hardj1
                     ptmax2=ptmax1
                     hardj1=i
                     ptmax1=pt(p(0,i))
                  elseif(pt(p(0,i)).gt.ptmax2) then
                     hardj2=i
                     ptmax2=pt(p(0,i))
                  endif
c                  write (*,*) hardj1,hardj2,ptmax1,ptmax2
               endif
            enddo
            
C-- NOW APPLY THE CUT I            
            
            if (abs(eta(p(0,hardj1))) .lt. xetamin
     &       .or.abs(eta(p(0,hardj2))) .lt. xetamin
     &       .or.eta(p(0,hardj1))*eta(p(0,hardj2)) .gt.0d0) then
             passcuts=.false.
             return
            endif
            
C-- NOW APPLY THE CUT II
            
            if (abs(eta(p(0,hardj1))-eta(p(0,hardj2))) .lt. deltaeta) then
             passcuts=.false.
             return
            endif
         
c            write (*,*) hardj1,hardj2,eta(p(0,hardj1)),eta(p(0,hardj2))
         
         ENDIF

C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>     
C     Event reweighting for matching. 
C     Also sets renormalization and factorization scales
C<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      rwgt=rewgt(P)
      if(rwgt.le.0d0)then
         passcuts=.false.
         return
      endif
         

C...Set strong couplings if event passed cuts
      if(.not.fixed_ren_scale.or..not.fixed_couplings) then
         if (.not.fixed_couplings)then
            do i=0,3
               do j=1,nexternal
                  PP(i,j)=p(i,j)
               enddo
            enddo
         endif
         call setpara('param_card.dat',.false.)
      endif

      IF (FIRSTTIME2) THEN
         FIRSTTIME2=.FALSE.
         write(6,*) 'alpha_s for scale ',scale,' is ', G**2/(16d0*atan(1d0))
      ENDIF

      RETURN
      END

