      subroutine set_ren_scale(P,rscale)
c----------------------------------------------------------------------
c     This is the USER-FUNCTION to calculate the renormalization
c     scale on event-by-event basis.
c----------------------------------------------------------------------      
      implicit none
      integer    maxexternal
      parameter (maxexternal=15)
      real*8 Pi
      parameter( Pi = 3.14159265358979323846d0 )
      real*8   alphas
      external alphas
c
c     INCLUDE and COMMON
c
      include 'genps.inc'
      include 'coupl.inc'

      integer    maxflow, i
      parameter (maxflow=999)
      integer idup(nexternal,maxproc)
      integer mothup(2,nexternal,maxproc)
      integer icolup(2,nexternal,maxflow)
      include 'leshouche.inc'
      include 'run.inc'

      double precision pmass(nexternal)
      common/to_mass/  pmass

      real*8 xptj,xptb,xpta,xptl
      real*8 xetamin,xqcut,deltaeta
      common /to_specxpt/xptj,xptb,xpta,xptl,xetamin,xqcut,deltaeta

      logical first
      data first/.true./

c
c     ARGUMENTS
c      
      REAL*8 P(0:3,maxexternal)
      REAL*8 rscale
c
c     EXTERNAL
c
      REAL*8 R2,DOT,ET,ETA,DJ,SumDot,PT

c----------
c     start
c----------

      if(fixed_ren_scale) then
         rscale=scale
      else

      if(ickkw.gt.0.or.xqcut.gt.0)then
c     alpha_s reweighted due to clustering in reweight.f
         rscale=scale
         return
      endif

      rscale=0d0

      if(first) then
         write(*,*) 'Using event- by event renormalization/factorization scale:'
         write(*,*) 'scalefact^2*(Max of squared masses of final-state particles + '
         write(*,*) '             sum of pT^2 for jets and massless particles)'
      endif
      do i=3,nexternal
         rscale=max(rscale,pmass(i)**2)
      enddo
      do i=3,nexternal
         if(iabs(idup(i,1)).le.5.or.idup(i,1).eq.21.or.pmass(i).eq.0d0)then
            rscale=rscale+pt(p(0,i))**2
         endif
      enddo

      rscale=sqrt(rscale)

      if(first)then
         write(*,*) 'mu_R = ',scalefact,' * ',rscale
         first=.false.
      endif

      rscale=rscale*scalefact

c
c-some examples of dynamical scales
c

c---------------------------------------
c-- total transverse energy of the event 
c---------------------------------------
c     scale=0d0
c     do i=3,nexternal
c      scale=scale+et(P(0,i))
c     enddo

c--------------------------------------
c-- scale^2 = \sum_i  (pt_i^2+m_i^2)  
c--------------------------------------
c     scale=0d0
c     do i=3,nexternal
c      scale=scale+pt(P(0,i))**2+dot(p(0,i),p(0,i))
c     enddo
c     scale=dsqrt(scale)

c--------------------------------------
c-- \sqrt(s): partonic energy
c--------------------------------------
c     scale=dsqrt(2d0*dot(P(0,1),P(0,2)))

      endif                     ! fixed_ren_scale

      G = SQRT(4d0*PI*ALPHAS(rscale))
      
      return
      end


      subroutine set_fac_scale(P,q2fact)
c----------------------------------------------------------------------
c     This is the USER-FUNCTION to calculate the factorization 
c     scales^2 on event-by-event basis.
c----------------------------------------------------------------------      
      implicit none
      integer    maxexternal
      parameter (maxexternal=15)
c
c     INCLUDE and COMMON
c
      include 'genps.inc'
      include 'coupl.inc'
      include 'message.inc'
c--masses and poles
c
c     ARGUMENTS
c      
      REAL*8 P(0:3,maxexternal)
      real*8 q2fact(2)
c
c     EXTERNAL
c
      REAL*8 R2,DOT,ET,ETA,DJ,SumDot,PT
c
c     LOCAL
c
      integer i
      logical first
      data first/.true./

c----------
c     start
c----------
      
      
      q2fact(1)=0d0             !factorization scale**2 for pdf1

      call set_ren_scale(P,q2fact(1))

      if(first)then
        write(*,*) 'mu_F = mu_R'
        first=.false.
      endif
      
      if (btest(mlevel,3)) then
        write(*,*)'setscales.f: Setting fact scale to ',q2fact(1)
      endif

      q2fact(1)=q2fact(1)**2

      q2fact(2)=q2fact(1)       !factorization scale**2 for pdf2
      

c
c-some examples of dynamical scales
c

c---------------------------------------
c-- total transverse energy of the event 
c---------------------------------------
c     q2fact(1)=0d0
c     do i=3,nexternal
c      q2fact(1)= q2fact(1)+et(P(0,i))**2
c     enddo
c     q2fact(2)=q2fact(1)  

c--------------------------------------
c-- scale^2 = \sum_i  (pt_i^2+m_i^2)  
c--------------------------------------
c     q2fact(1)=0d0
c     do i=3,nexternal
c      q2fact(1)=q2fact(1)+pt(P(0,i))**2+dot(p(0,i),p(0,i))
c     enddo
c     q2fact(2)=q2fact(1)  

c--------------------------------------
c-- \sqrt(s): partonic energy
c--------------------------------------
c     q2fact(1)=2d0*dot(P(0,1),P(0,2))
c     q2fact(2)=q2fact(1)  

      
      return
      end


